@@ -10,12 +10,13 @@
 """
 from collections import namedtuple
 import time
+import warnings
 
 from blist import sortedlist
 
 
 __all__ = ['Gauge', 'Momentum']
-__version__ = '0.0.4'
+__version__ = '0.0.5'
 
 
 add = 1
@@ -45,22 +46,6 @@ class Gauge(object):
         self.momenta = sortedlist(key=indexer(2))
         self._plan = sortedlist(key=indexer(0))
 
-    @property
-    def max(self):
-        return self._max
-
-    @max.setter
-    def max(self, max):
-        self.set_max(max)
-
-    @property
-    def min(self):
-        return self._min
-
-    @min.setter
-    def min(self, min):
-        self.set_min(min)
-
     @property
     def determination(self):
         """Returns the cached determination. If there's no the cache, it
@@ -85,10 +70,26 @@ class Gauge(object):
         except AttributeError:
             pass
 
+    @property
+    def max(self):
+        return self._max
+
+    @max.setter
+    def max(self, max):
+        self.set_max(max)
+
+    @property
+    def min(self):
+        return self._min
+
+    @min.setter
+    def min(self, min):
+        self.set_min(min)
+
     def _set_limits(self, min=None, max=None, limit=True, at=None):
         if limit:
             at = now_or(at)
-            value = self.current(at)
+            value = self.get(at)
         if min is not None:
             self._min = min
         if max is not None:
@@ -106,9 +107,23 @@ class Gauge(object):
         del self.determination
 
     def set_max(self, max, limit=True, at=None):
+        """Changes the maximum.
+
+        :param max: the value to set as the maximum.
+        :param limit: limits the current value to be below the new maximum.
+                      (default: ``True``)
+        :param at: the time to change. (default: now)
+        """
         self._set_limits(max=max, limit=limit, at=at)
 
     def set_min(self, min, limit=True, at=None):
+        """Changes the minimum.
+
+        :param min: the value to set as the minimum.
+        :param limit: limits the current value to be above the new minimum.
+                      (default: ``True``)
+        :param at: the time to change. (default: now)
+        """
         self._set_limits(min=min, limit=limit, at=at)
 
     def _current_value_and_velocity(self, at=None):
@@ -127,7 +142,7 @@ class Gauge(object):
         velocity = delta / (next_time - prev_time)
         return (value, velocity)
 
-    def current(self, at=None):
+    def get(self, at=None):
         """Predicts the current value.
 
         :param at: the time to observe. (default: now)
@@ -152,7 +167,7 @@ class Gauge(object):
         :raises ValueError: the value is out of the range.
         """
         at = now_or(at)
-        value = self.current(at) + delta
+        value = self.get(at) + delta
         if limit:
             if delta > 0 and value > self.max:
                 raise ValueError(
@@ -188,7 +203,24 @@ class Gauge(object):
         :raises ValueError: the value is out of the range.
         """
         at = now_or(at)
-        return self.incr(value - self.current(at), limit, at)
+        return self.incr(value - self.get(at), limit, at)
+
+    def when(self, value):
+        """When the gauge reaches to the goal value.
+
+        :param value: the goal value.
+
+        :raises ValueError: the gauge will not reach to the goal value.
+        """
+        if self.determination:
+            determination = self.determination
+            if determination[0][1] == value:
+                return determination[0][0]
+            for prev, next in zip(determination[:-1], determination[1:]):
+                if prev[1] < value <= next[1] or prev[1] > value >= next[1]:
+                    t = (value - prev[1]) / float(next[1] - prev[1])
+                    return prev[0] + (next[0] - prev[0]) * t
+        raise ValueError('The gauge will not reach to {0}'.format(value))
 
     def add_momentum(self, velocity_or_momentum, since=None, until=None):
         """Adds a momentum. A momentum includes the velocity and the times to
@@ -225,6 +257,18 @@ class Gauge(object):
         self.momenta.remove(momentum)
         del self.determination
 
+    def clear_momenta(self, at=None):
+        """Removes all momenta. The value is set as the current value. The
+        determination would be changed.
+
+        :param at: the time base. (default: now)
+        """
+        at = now_or(at)
+        value = self.get(at)
+        del self.momenta[:]
+        self.set(value, False, at)
+        return value
+
     def forget_past(self, value=None, at=None):
         """Discards the momenta which doesn't effect anymore.
 
@@ -233,7 +277,7 @@ class Gauge(object):
         """
         at = now_or(at)
         if value is None:
-            value = self.current(at)
+            value = self.get(at)
         self.value = value
         self.set_at = at
         # forget past momenta
@@ -241,6 +285,7 @@ class Gauge(object):
         stop = self.momenta.bisect_left(Momentum(0, until=at))
         del self.momenta[start:stop]
         del self.determination
+        return value
 
     def determine(self):
         """Determines the transformations from the time when the value set to
@@ -316,13 +361,14 @@ class Gauge(object):
             # prepare the next iteration
             prev_time = time
             try:
-                time, method, momentum = self._plan[x]
+                while True:
+                    time, method, momentum = self._plan[x]
+                    if momentum in self.momenta:
+                        break
+                    del self._plan[x]
             except IndexError:
                 span = inf
                 continue
-            if momentum not in self.momenta:
-                del self._plan[x]
-                continue
             # normalize time
             if time is None:
                 time = self.set_at
@@ -344,12 +390,21 @@ class Gauge(object):
 
     def __repr__(self, at=None):
         at = now_or(at)
-        current = self.current(at)
+        value = self.get(at)
         if self.min == 0:
             fmt = '<{0} {1:.2f}/{2}>'
         else:
             fmt = '<{0} {1:.2f} between {3}~{2}>'
-        return fmt.format(type(self).__name__, current, self.max, self.min)
+        return fmt.format(type(self).__name__, value, self.max, self.min)
+
+    # deprecated features
+
+    def current(self, at=None):
+        # deprecated since v0.0.5
+        warnings.warn(DeprecationWarning('Use Gauge.get() instead'))
+        return self.get(at)
+
+    current.__doc__ = get.__doc__
 
 
 class Momentum(namedtuple('Momentum', ['velocity', 'since', 'until'])):
